import GLib from 'gi://GLib';
import Gio from 'gi://Gio';
import { Extension } from 'resource:///org/gnome/shell/extensions/extension.js';
import * as Main from 'resource:///org/gnome/shell/ui/main.js';
import * as Log from './modules/log.js';
import * as Panel from './modules/panel.js';

export default class SuperGfxExtension extends Extension {
    gpuModeIndicator = null;
    quickToggles = null;
    systemMenu = null;

    enable() {
        if (Main.panel.statusArea.quickSettings._system)
            return this.enableSystemMenu();
        return this.enableSystemMenuQueued();
    }

    enableSystemMenuQueued() {
        var success = false;
        GLib.idle_add(GLib.PRIORITY_DEFAULT, () => {
            if (!Main.panel.statusArea.quickSettings._system)
                return GLib.SOURCE_CONTINUE;
            success = this.enableSystemMenu();
            return GLib.SOURCE_REMOVE;
        });
        return success;
    }

    enableSystemMenu() {
        Gio.Resource.load(super.path +
            '/resources/org.gnome.Shell.Extensions.supergfxctl-gex.gresource')._register();
        this.systemMenu = Main.panel.statusArea.quickSettings;
        if (typeof this.systemMenu == 'undefined') {
            Log.raw('init', 'system menu is not defined');
            return false;
        }
        this.gpuModeIndicator = new Panel.GpuModeIndicator();
        this.quickToggles = new Panel.GpuModeToggle(this.gpuModeIndicator, super.path, this);
        this.gpuModeIndicator.quickSettingsItems.push(this.quickToggles);
        this.systemMenu._indicators.remove_child(this.systemMenu._system);
        this.systemMenu._indicators.add_child(this.gpuModeIndicator);
        this.systemMenu._indicators.add_child(this.systemMenu._system);
        this.systemMenu._addItems(this.gpuModeIndicator.quickSettingsItems);

        for (const item of this.gpuModeIndicator.quickSettingsItems) {
            this.systemMenu.menu._grid.set_child_below_sibling(item, this.systemMenu._backgroundApps.quickSettingsItems[0]);
        }
        return true;
    }

    disable() {
        this.quickToggles?.disable();
        this.gpuModeIndicator?.destroy();
        this.quickToggles = null;
        this.gpuModeIndicator = null;
    }
}

export function init() {
    return new SuperGfxExtension();
}
